{ ------------------------------------------------------------------------------
  Credits and thanks to:
  Craig Ward, 100554.2072@compuserve.com for code on logging exceptions
  Brad Stowers, bstowers@pobox.com, for the code on OnShowHint
  Marco Cantu, marcocantu@compuserve.com for writing Mastering Delphi 3!
  ------------------------------------------------------------------------------
  I took several useful components and consolidated them into one very useful
  component.  Please note that this component must be listed first in the
  component creation order

  TApplicationProperties, Version 2., June 21, 1997
  Designed for use with Delphi 2.0 or later and Windows95
  ------------------------------------------------------------------------------
  Copyright 1997, Kenneth R Kyler dba YakWorks Software.  All Rights Reserved.
  This component can be freely used and distributed in commercial and private
  environments, provied this notice is not modified in any way.
  E-Mail: Ken@Kyler.com
  ------------------------------------------------------------------------------
  How to use:
     First, start by looking at the sample program!

     Where you see "<= Press F1 to see the definition" position the cursor over
     the word in question and press F1. The Delphi Help tpoic will explain the
     use of the property/method/event in question.

     Methods:
        ErrorLog    <= The filename of the log where exceptions are written.
                       LogException must be true.
        HintColor   <= Press F1 to see the definition
        HintPause   <= Press F1 to see the definition
        LogException   <= if true, then LogIt can write to the ErrorLog
        ShowException  <= if true, exceptions dialog blocks are displayed to
                          the user. If false, exceptions are silent.
        ShowHint  <= Press F1 to see the definition

     Properties:
        Loaded <= Press F1 to see the definition
        LogIt(sText: string; E: Exception); <= Pass a string to be written to
              the log file specified in ErrorLog as well as the exception that
              was triggered. Usually used in OnException but can also be used
              in any try..except..end block
        
     Events: **** Press F1 to see the definition of these events
        OnActivate
        OnDeactivate
        OnException
        OnHelp
        OnHint
        OnIdle
        OnMessage
        OnMinimize
        OnShowHint * Note that I've allowed the programmer to override the
                     default event programatically
  -----------------------------------------------------------------------------}
unit AppProp;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf, MMSystem;

type

  TDuplicateError = class(Exception);

	TFilenameProperty = class(TStringProperty)
	public
   	{ Public Declarations }
		procedure Edit; override;
		function GetAttributes: TPropertyAttributes; override;
	end;

  TApplicationProperties = class(TComponent)
  private
     FHint: String;
     FHintColor: TColor;
     FHintPause,
     FHintHidePause,
     FHintShortPause: Integer;
     FShowHint: Boolean;
     StoreFileName : boolean;

     F0nActivate: TNotifyEvent;
     F0nDeactivate: TNotifyEvent;
     F0nException: TExceptionEvent;
     F0nHelp: THelpEvent;
     F0nHint: TNotifyEvent;
     F0nIdle: TIdleEvent;
     F0nMessage: TMessageEvent;
     FOnMinimize: TNotifyEvent;
     FOnRestore: TNotifyEvent;
     FOnShowHint: TShowHintEvent;

     FFilename: TFileName;
     FShowExcep: Boolean;
     FLogIt: boolean;
     procedure SetLogIt(Value: boolean);
     procedure SetShowHint(Value: boolean);
     procedure SetHintPause(Value: integer);
     procedure SetHintHidePause(Value: integer);
     procedure SetHintShortPause(Value: integer);
     procedure SetHintColor(Value: TColor);
     procedure SetFileName(Value : TFilename);
     procedure SetShowExcep(Value: Boolean);
     // the default event handlers
     procedure DoHintInfo(var HintStr: string; var CanShow: boolean;
               var HintInfo: THintInfo);
  protected
     {Protected declarations}
     procedure Loaded;  override;
     procedure MyMinimize(Sender: TObject);
     Procedure MyRestore(Sender: TObject);
  public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure LogIt(sText: string; E: Exception);     
  published
     // AppProp properties
     property Hint: String
        read FHint write FHint;
     property HintColor: TColor
        read FHintColor write SetHintColor default clInfoBk;
     property HintPause: Integer
        read FHintPause write SetHintPause default 2500;
     property HintHidePause: integer
        read FHintHidePause write FHintHidePause default 2500;
     property HintShortPause: integer
        read FHintShortPause write SetHintShortPause default 50;
     property ShowHint: Boolean
        read FShowHint write SetShowHint default true;
     property OnActivate: TNotifyEvent
        read F0nActivate write F0nActivate;
     property OnDeactivate: TNotifyEvent
        read F0nDeactivate write F0nDeactivate;
     property OnException: TExceptionEvent
        read F0nException write F0nException;
     property OnHelp: THelpEvent
        read F0nHelp write F0nHelp;
     property OnHint: TNotifyEvent
        read F0nHint write F0nHint;
     property OnIdle: TIdleEvent
        read F0nIdle write F0nIdle;
     property OnMessage: TMessageEvent
        read F0nMessage write F0nMessage;
     property OnMinimize: TNotifyEvent
        read FOnMinimize write FOnMinimize;
     property OnRestore: TNotifyEvent
        read FOnRestore write FOnRestore;

     property OnShowHint: TShowHintEvent
        read FOnShowHint write FOnShowHint;

     property ErrorLog: string
        read FFilename write SetFilename stored StoreFileName;
     property ShowException: Boolean
        read FShowExcep write SetShowExcep default True;
     property LogException: boolean
        read FLogIt write SetLogIt default False;
  end;

procedure Register;

implementation

const
  CRLF = #13#10;


procedure Register;
begin
  RegisterComponents('YakWorks', [TApplicationProperties]);
  RegisterPropertyEditor(TypeInfo(TFileName), TApplicationProperties, 'ErrorLog',
		TFilenameProperty);
end;

constructor TApplicationProperties.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FShowExcep := true;
  FShowHint := true;
  FLogIt := false;
  FHintColor := clInfoBk;
  FHintPause := 2500;
  FHintHidePause := 2500;
  FHintShortPause := 50;
  FFilename := 'Error.log';
  StoreFileName := FALSE;
end;

destructor TApplicationProperties.Destroy;
begin
  inherited Destroy;
end;

procedure TApplicationProperties.Loaded;
begin
  if FHint <> '' then
     Application.Hint := FHint;
  if FHintColor <> Application.HintColor then
     Application.HintColor := FHintColor;
  if FHintPause <> Application.HintPause then
     Application.HintPause := FHintPause;
  if FShowHint <> Application.ShowHint then
     Application.ShowHint := FShowHint;
  if FHintHidePause <> Application.HintHidePause then
     Application.HintHidePause := FHintHidePause;
  if FHintShortPause <> Application.HintShortPause then
     Application.HintShortPause := FHintShortPause;

  if Assigned( FOnShowHint) then
     Application.OnShowHint := FOnShowHint // allows the programmer to write a local event
  else
     Application.OnShowHint := DoHintInfo;
  if Assigned( F0nActivate) then
     Application.OnActivate := F0nActivate;
  if Assigned( F0nDeactivate) then
     Application.OnDeactivate := F0nDeactivate;
  if Assigned( F0nException) then
     Application.OnException := F0nException;
  if Assigned( F0nHelp) then
     Application.OnHelp := F0nHelp;
  if Assigned( F0nHint ) then
     Application.OnHint := F0nHint;
  if Assigned( F0nIdle) then
     Application.OnIdle := F0nIdle;
  if Assigned( F0nMessage) then
     Application.OnMessage := F0nMessage;

  Application.OnMinimize := MyMinimize;
  Application.OnRestore := MyRestore;
end;

procedure TApplicationProperties.SetShowHint(Value: boolean);
begin
  if (FShowHint <> Value) then begin
     FShowHint := Value;
     Application.ShowHint := Value;
  end;
end;

procedure TApplicationProperties.SetHintColor(Value: TColor);
begin
  if (FHintColor <> Value) then begin
     FHintColor := Value;
     Application.HintColor := FHintColor;
  end;
end;

procedure TApplicationProperties.SetHintPause(Value: integer);
begin
   if (FHintPause <> Value) then begin
     FHintPause := Value;
     Application.HintPause := FHintPause;
   end;
end;

procedure TApplicationProperties.SetHintHidePause(Value: integer);
begin
   if (FHintHidePause <> Value) then begin
     FHintHidePause := Value;
     Application.HintHidePause := FHintHidePause;
   end;
end;

procedure TApplicationProperties.SetHintShortPause(Value: integer);
begin
   if (FHintShortPause <> Value) then begin
     FHintShortPause := Value;
     Application.HintShortPause := FHintShortPause;
   end;
end;

procedure TApplicationProperties.SetLogIt(Value: boolean);
begin
 if (FLogIt <> Value) then FLogIt := Value;
end;


{set text file}
procedure TApplicationProperties.SetFilename(Value: TFilename);
begin
  if (FFilename <> Value) then begin
     FFilename := Value;
     StoreFilename := FFilename <> Application.ExeName;
  end;
end;

procedure TApplicationProperties.SetShowExcep(Value: boolean);
begin
 if (FShowExcep <> Value) then FShowExcep := Value;
end;

{write exception message to text file}
procedure TApplicationProperties.LogIt(sText: string; E: Exception);
var
 tF: TextFile;
begin
  if FLogIt then begin
     AssignFile(tF,FFilename);                        {assign field to Text-File}
     if FileExists( FFilename ) then Append(tF)       {prepare to append to file}
     else Rewrite( tF );
     WriteLn(tF,FormatDateTime('c', Now));
     WriteLn(tF,' ');
     if (sText <> '') then begin
        WriteLn(tF, sText);
        WriteLn(tF,' ');
     end;
     WriteLn(tF, E.Message);
     CloseFile(tF);
  end;                                      {close file}

  if FShowExcep then Application.ShowException(E); {show exception if reqd}
end;

procedure TFilenameProperty.Edit;
var OpenDialog : TOpenDialog;
begin
	OpenDialog := TOpenDialog.create(Application);
	try
   	OpenDialog.filename := GetValue;
		OpenDialog.filter := 'Text Files (*.txt)|*.txt|' +
                          'Log Files (*.log)|*.log|' +
                          'All Files (*.*)|*.*';
   	OpenDialog.options := [ofFileMustExist, ofPathMustExist];
   	if OpenDialog.execute then
			SetValue(OpenDialog.Filename);
	finally
		OpenDialog.free;
	end;
end;

function TFilenameProperty.GetAttributes : TPropertyAttributes;
begin
  Result := [paDialog];
end;

procedure TApplicationProperties.DoHintInfo(var HintStr: string;
  var CanShow: boolean; var HintInfo: THintInfo);
var
  II: TIconInfo;
  Bmp: Windows.TBitmap;
begin
  // all of this is courtesy of Brad Stowers
  with HintInfo do begin
     // Make sure we have a control that fired the hint.
     if HintControl = NIL then exit;
     // Convert the cursor's coordinates from relative to hint to
     // relative to screen.
     HintPos := HintControl.ClientToScreen(CursorPos);
     // Get some information about the cursor that is used for the
     // hint control
     GetIconInfo(Screen.Cursors[HintControl.Cursor], II);
     // Get some information about the bitmap representing the cursor
     GetObject(II.hbmMask, SizeOf(Windows.TBitmap), @Bmp);
     // If the info did not include a color bitmap then the mask bitmap
     // is really two bitmaps, an AND & XOR mask.  Increment our Y
     // position by the bitmap's height.
     if II.hbmColor = 0 then
        inc(HintPos.Y, Bmp.bmHeight div 2)
     else
        inc(HintPos.Y, Bmp.bmHeight);
     // Subtract out the Y hotspot position
     dec(HintPos.Y, II.yHotSpot);
     // We are responsible for cleaning up the bitmap handles returned
     // by GetIconInfo.
     DeleteObject(II.hbmMask);
     DeleteObject(II.hbmColor);
  end;
end;

procedure TApplicationProperties.MyMinimize(Sender: TObject);
begin
  // this is courtsey of Marco Cantu as written in Mastering Delphi 3
  PlaySound('Minimize', 0, Snd_ASync);
  if Assigned( FOnMinimize ) then
     FOnMinimize(Self);
end;

procedure TApplicationProperties.MyRestore(Sender: TObject);
begin
  // this is courtsey of Marco Cantu as written in Mastering Delphi 3
  PlaySound('RestoreUp', 0, Snd_ASync);
  if Assigned( FOnRestore ) then
     FOnRestore(Self);
end;

end.
// da end!
